import numpy as np


def normalize(array: np.array, lower: float, upper: float) -> np.array:
    newarray = array

    newarray += (0 - np.min(newarray))
    newarray *= ((upper - lower) / np.amax(newarray))
    newarray += lower

    return newarray


def denoise(array: np.array, lower=None, upper=None) -> np.array:
    newarray = array

    if lower:
        newarray[newarray < lower] = lower
        if upper:
            newarray[newarray > upper] = lower
    elif upper:
        newarray[newarray > upper] = 0.

    return newarray


def pad(array: np.array, shape: (int, int, int)) -> np.array:
    newarray = array

    for array_d, output_d in zip(newarray.shape, shape):
        if array_d > output_d:
            raise ValueError('Desired dimensions must be greater than input dimensions')


def rotate_image(array: np.array) -> np.array:
    return np.flip(array, axis=(0, 1))



import numpy as np


def align(array, reader):
    nifti_array = array
    alignment_matrix = [reader.GetMetaData('srow_x').split()[:3], reader.GetMetaData('srow_y').split()[:3],
                        reader.GetMetaData('srow_z').split()[:3]]

    for row in range(len(alignment_matrix)):
        for col in range(len(alignment_matrix[0])):
            alignment_matrix[row][col] = int(round(float(alignment_matrix[row][col])))

    flip_list = [sum(x) for x in zip(alignment_matrix[0], alignment_matrix[1], alignment_matrix[2])]

    for position in range(len(flip_list)):
        if flip_list[position] < 0:
            nifti_array = np.flip(nifti_array, axis=position)

    for row in range(len(alignment_matrix)):
        for col in range(len(alignment_matrix[0])):
            alignment_matrix[row][col] = abs(alignment_matrix[row][col])

    if abs(alignment_matrix[0][0]) != 1:
        tranpose_list = [0, 1, 2]
        for index in range(len(alignment_matrix)):
            if alignment_matrix[index][0] == 1:
                swap_index = index
                break
        tranpose_list[swap_index], tranpose_list[0] = tranpose_list[0], tranpose_list[swap_index]
        alignment_matrix[swap_index], alignment_matrix[0] = alignment_matrix[0], alignment_matrix[swap_index]

        nifti_array = np.transpose(nifti_array, tranpose_list)
        nifti_array = np.flip(nifti_array, axis=swap_index)

    if abs(alignment_matrix[1][1]) != 1:
        tranpose_list = [0, 1, 2]
        for index in range(len(alignment_matrix)):
            if alignment_matrix[index][1] == 1:
                swap_index = index
                break
        tranpose_list[swap_index], tranpose_list[1] = tranpose_list[1], tranpose_list[swap_index]
        alignment_matrix[swap_index], alignment_matrix[1] = alignment_matrix[1], alignment_matrix[swap_index]

        nifti_array = np.transpose(nifti_array, tranpose_list)
        nifti_array = np.flip(nifti_array, axis=swap_index)

    if abs(alignment_matrix[2][2]) != 1:
        tranpose_list = [0, 1, 2]
        for index in range(len(alignment_matrix)):
            if alignment_matrix[index][2] == 1:
                swap_index = index
                break
        tranpose_list[swap_index], tranpose_list[2] = tranpose_list[2], tranpose_list[swap_index]
        alignment_matrix[swap_index], alignment_matrix[2] = alignment_matrix[2], alignment_matrix[swap_index]

        nifti_array = np.transpose(nifti_array, tranpose_list)
        nifti_array = np.flip(nifti_array, axis=swap_index)

    return nifti_array



import argparse
import os

from .nifti import Nifti


def parsercli():
    parser = argparse.ArgumentParser(prog='niftiprocessing')

    # Input
    parser.add_argument('path', type=str, help='Path to file')

    # Input options
    parser.add_argument('--precision', type=int, choices=[32, 64], default=32, help='Sets datatype precision in float32 or 64')

    # Information
    parser.add_argument('--info', '-i', action='store_true', help='Outputs .nii file metadata')
    parser.add_argument('--plot', '-p', type=int, help='Plots image of 2nd/3rd dimension in .nii file (Requires MatplotLib)')

    # Tools
    parser.add_argument('--align', '-a', action='store_true', help='Orients .nii file in MNI152 space')

    parser.add_argument('--normalize', '-n', action='store_true', help='Normalizes .nii file values within set values (Default: 0.,1.)')
    parser.add_argument('--normalize-lower', type=checkintfloat, default=0.0, help='Lower value for normalization (Default: 0.)')
    parser.add_argument('--normalize-upper', type=checkintfloat, default=1.0, help='Lower value for normalization (Default: 0.)')

    parser.add_argument('--rotate', '-r', action='store_true', help='Rotates .nii file by 90 degress in 1st and 2nd dimension')

    parser.add_argument('--output', '-o', action='store_true', help='Filename for .nii file generation (Note: Currently erases orginal metadata)')

    args = parser.parse_args()

    # Initialize Nifti
    niftiobject = Nifti(args.path, args.precision)

    if args.info:
        niftiobject.information()

    if args.align:
        niftiobject.align_nifti()

    if args.normalize:
        niftiobject.normalize_nifti(args.normalize_lower, args.normalize_upper)

    if args.rotate:
        niftiobject.rotate_nifti()

    if args.plot:
        niftiobject.plot(args.plot)

    if args.output:
        niftiobject.write_nifti()


def checkintfloat(x):
    if not (type(x) == float or type(x) == int):
        TypeError('Input(s) for --normalize must be an int or float')

    return float(x)


def main():
    parsercli()



import niftiprocessing
import sys
import os

# Debugging
if __name__ == '__main__':
    test_filepath = os.path.normpath(os.getcwd()).split(os.sep)

    test_filepath = os.path.join(os.sep, *test_filepath[:-1], 'test_directory/004101-1-1.nii.gz')
    # test_filepath = os.path.join(os.sep, *test_filepath[:-1], 'test_directory/test.nii.gz')

    sys.argv = ['niftiproc', test_filepath, '-i', '-a']

    niftiprocessing.main()



import numpy as np

try:
    import matplotlib.pyplot as plt
except ImportError:
    print('For external plotting, matplotlib>=3.0 must be installed')


def plot_slice(data_array: np.ndarray, slice_index: int, time_index: int = 0, step_size: int = 10, figsize=(10, 9),
               vplots: int = 1, hplots: int = 1, cmap: str = 'gray'):
    """Plot MRI/fMRI slice(s) in matplotlib

    Plot image(s) of MRI/fMRI slice(s) in matplotlib

    Args:
        data_array: A numpy array containing data points for an MRI/fMRI scan
        slice_index: An integer specifying index of slice to plot
        time_index: Optional; An integer specifying time index ([0]) of slice to plot in fMRI data arrays
        step_size: Optional; An integer specyfying number of indices to skip in data array when plotting two or more
            slices
        figsize: Optional; A tuple specyfing matplotlib figure size
        vplots: Optional; An integer specyfying number of vertical plots in matplotlib figure
        hplots: Optional; An integer specyfying number of horizontal plots in matplotlib figure
        cmap: Optional; A string specifying matplotlib plot cmap(s)
    """
    plt.figure(figsize=figsize)

    if data_array.ndim == 3 or (data_array.ndim == 4 and data_array.shape[-1] <= 3):
        for position in range(vplots * hplots):
            plt.subplot(vplots, hplots, position + 1)
            plt.xticks([])
            plt.yticks([])
            plt.imshow(data_array[slice_index + (position * step_size)], cmap=cmap)
            plt.title(f'Slice index: {slice_index + (position * step_size)}')
    elif data_array.ndim == 4 or data_array.ndim == 5:
        for position in range(vplots * hplots):
            plt.subplot(vplots, hplots, position + 1)
            plt.xticks([])
            plt.yticks([])
            plt.imshow(data_array[time_index][slice_index + (position * step_size)], cmap=cmap)
            plt.title(f'Time index: {time_index}, Slice index: {slice_index + (position * step_size)}')
    else:
        raise ValueError('Input data must be a 3d, 4d, or 5d image')

    plt.show()


import setuptools

with open("README.md", "r", encoding="utf-8") as fh:
    long_description = fh.read()

setuptools.setup(
    name="niftiprocessing",
    version="0.0.5",
    author="Alan Lee",
    author_email="alleetw101@gmail.com",
    description="A nifti processing tool",
    long_description=long_description,
    long_description_content_type="text/markdown",
    url="https://github.com/alleetw101/niftiprocessing",
    classifiers=[
        "Programming Language :: Python :: 3",
        "License :: OSI Approved :: MIT License",
        "Operating System :: OS Independent",
    ],
    install_requires=[
        'SimpleITK>=2.0',
        'numpy>=1.0'
    ],
    entry_points={
        'console_scripts': ['niftiprocessing=niftiprocessing:main'],
    },
    packages=setuptools.find_packages(),
    python_requires='>=3.6',
)



import os
import SimpleITK as sitk

from .numpyhelper import normalize, rotate_image
from .graphical import plot_slice
from .niftihelper import align


class Nifti:

    def __init__(self, filepath: str, precision: int):
        dtype = sitk.sitkFloat32 if precision == 32 else sitk.sitkFloat64

        sitk_image = sitk.ReadImage(filepath, dtype)

        self.nifti_array = sitk.GetArrayFromImage(sitk_image)
        self.file_name = os.path.basename(filepath)
        self.filepath = filepath
        self.reader = sitk.ImageFileReader()

    def information(self):
        print(f'--{self.file_name}--')
        print(f'Shape: {self.nifti_array.shape}')

        self.reader.SetFileName(self.filepath)
        self.reader.LoadPrivateTagsOn()

        self.reader.ReadImageInformation()

        for k in self.reader.GetMetaDataKeys():
            v = self.reader.GetMetaData(k)
            if not (v == '0' or v == ''):
                print(f"({k}) == \"{v}\"")

    def plot(self, slice_index: int):
        plot_slice(self.nifti_array, slice_index)

    def align_nifti(self):
        self.nifti_array = align(self.nifti_array, self.reader)

    def normalize_nifti(self, lower: float, upper: float):
        self.nifti_array = normalize(self.nifti_array, lower, upper)

    def rotate_nifti(self):
        self.nifti_array = rotate_image(self.nifti_array)

    def write_nifti(self):
        output = sitk.GetImageFromArray(self.nifti_array)

        output_path = os.path.join(os.getcwd(), 'test.nii.gz')

        sitk.WriteImage(output, output_path)